package com.io.yutian.elementoriginlib.datasync;

import com.io.yutian.elementoriginlib.mongodb.MongoDBIO;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.model.ReplaceOptions;
import org.bson.Document;
import org.bukkit.entity.Player;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.exceptions.JedisException;

public class SyncModule {
    public static final String KEY_PREFIX = "playerdata";
    private static final int DEFAULT_LOCK_EXPIRE = 15;
    private static final String _ID = "_id";

    private final String moduleName;
    private final String databaseName;
    private final String collectionName;

    public SyncModule(String moduleName, String databaseName, String collectionName) {
        this.moduleName = moduleName;
        this.databaseName = databaseName;
        this.collectionName = collectionName;
    }

    public SyncResult syncToRedis(Player player, JedisPool jedisPool, MongoDBIO mongoDBIO) {
        if (player == null || jedisPool == null || mongoDBIO == null) {
            return SyncResult.INVALID_PARAMETERS;
        }

        String playerId = player.getUniqueId().toString();
        String redisKey = buildRedisKey(playerId);
        String lockKey = redisKey + ":lock";

        try (Jedis jedis = jedisPool.getResource()) {
            if (!acquireLock(jedis, lockKey)) {
                return SyncResult.LOCK_FAILED;
            }
            MongoCollection<Document> collection = mongoDBIO.getMongoDatabase()
                    .getCollection(collectionName);
            Document query = new Document(_ID, player.getUniqueId());
            Document mongoDoc = collection.find(query).first();

            if (mongoDoc == null) {
                return SyncResult.MONGO_DATA_NOT_FOUND;
            }

            String jsonData = mongoDoc.toJson();
            jedis.set(redisKey, jsonData);
            return SyncResult.SUCCESS;
        } catch (JedisException e) {
            return SyncResult.REDIS_ERROR;
        } catch (Exception e) {
            return SyncResult.MONGO_ERROR;
        } finally {
            releaseLock(jedisPool, lockKey);
        }
    }

    public SyncResult syncToMongoDB(Player player, JedisPool jedisPool, MongoDBIO mongoDBIO) {
        if (player == null || jedisPool == null || mongoDBIO == null) {
            return SyncResult.INVALID_PARAMETERS;
        }

        String playerId = player.getUniqueId().toString();
        String redisKey = buildRedisKey(playerId);
        String lockKey = redisKey + ":lock";

        try (Jedis jedis = jedisPool.getResource()) {
            if (!acquireLock(jedis, lockKey)) {
                return SyncResult.LOCK_FAILED;
            }

            if (!jedis.exists(redisKey)) {
                return SyncResult.NOT_FOUND_KEY;
            }

            String redisValue = jedis.get(redisKey);
            if (redisValue == null || redisValue.isEmpty()) {
                return SyncResult.EMPTY_DATA;
            }

            MongoCollection<Document> collection = mongoDBIO.getMongoDatabase()
                    .getCollection(collectionName);

            Document doc = Document.parse(redisValue);
            collection.replaceOne(
                    new Document(_ID, player.getUniqueId()),
                    doc,
                    new ReplaceOptions().upsert(true)
            );

            jedis.del(redisKey);
            return SyncResult.SUCCESS;
        } catch (JedisException e) {
            return SyncResult.REDIS_ERROR;
        } catch (Exception e) {
            return SyncResult.MONGO_ERROR;
        } finally {
            releaseLock(jedisPool, lockKey);
        }
    }

    public boolean checkMongoDBConnection(MongoDBIO mongoDBIO) {
        try {
            Document pingResult = mongoDBIO.getMongoDatabase()
                    .runCommand(new Document("ping", 1));
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    public boolean checkRedisConnection(JedisPool jedisPool) {
        try (Jedis jedis = jedisPool.getResource()) {
            String result = jedis.ping();
            return "PONG".equals(result);
        } catch (Exception e) {
            return false;
        }
    }

    private String buildRedisKey(String playerId) {
        return String.format("%s:%s:%s", KEY_PREFIX, collectionName, playerId);
    }

    private boolean acquireLock(Jedis jedis, String lockKey) {
        long result = jedis.setnx(lockKey, "1");
        if (result == 1) {
            jedis.expire(lockKey, DEFAULT_LOCK_EXPIRE);
            return true;
        }
        return false;
    }

    private void releaseLock(JedisPool jedisPool, String lockKey) {
        try (Jedis jedis = jedisPool.getResource()) {
            jedis.del(lockKey);
        } catch (JedisException e) {
        }
    }

    public String getModuleName() {
        return moduleName;
    }

    public String getDatabaseName() {
        return databaseName;
    }

    public String getCollectionName() {
        return collectionName;
    }
}