package com.io.yutian.elementoriginlib.mongodb;

import com.io.yutian.elementoriginlib.serialize.SerializeHelper;
import com.mongodb.BasicDBObject;
import com.mongodb.ConnectionString;
import com.mongodb.MongoClientSettings;
import com.mongodb.client.*;
import com.mongodb.client.model.Filters;
import com.mongodb.client.result.DeleteResult;
import com.mongodb.client.result.UpdateResult;
import org.bson.Document;
import org.bson.UuidRepresentation;
import org.bson.codecs.UuidCodec;
import org.bson.codecs.configuration.CodecRegistries;
import org.bson.codecs.configuration.CodecRegistry;
import org.bson.conversions.Bson;
import org.bson.types.ObjectId;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;

public class MongoSupport {
    private final static Logger LOG = LoggerFactory.getLogger(MongoSupport.class);
    public final static String ID = "_id";
    private final static Map<Class<?>, String> classCollectionName = new ConcurrentHashMap<>();
    private final static Map<String, MongoCollection<Document>> collections = new ConcurrentHashMap<>();

    private static final CodecRegistry CODEC_REGISTRY = CodecRegistries.fromRegistries(
            CodecRegistries.fromCodecs(new UuidCodec(UuidRepresentation.STANDARD)),
            MongoClientSettings.getDefaultCodecRegistry()
    );

    protected MongoDatabase mongoDatabase;

    public MongoSupport(String host, int port, String username, String password, String dbname) {
        String connectionString = String.format(
                "mongodb://%s:%s@%s:%d/?authSource=admin",
                URLEncoder.encode(username, StandardCharsets.UTF_8),
                URLEncoder.encode(password, StandardCharsets.UTF_8),
                host, port
        );
        MongoClientSettings settings = MongoClientSettings.builder()
                .applyConnectionString(new ConnectionString(connectionString))
                .codecRegistry(CODEC_REGISTRY)
                .uuidRepresentation(UuidRepresentation.STANDARD)
                .build();
        MongoClient client = MongoClients.create(settings);
        mongoDatabase = client.getDatabase(dbname);
        if (mongoDatabase == null) {
            throw new IllegalArgumentException("获取数据库实例失败：" + dbname);
        }
    }

    public MongoSupport(MongoClient mongo, String dbname) {
        mongoDatabase = mongo.getDatabase(dbname);
        if (mongoDatabase == null) {
            throw new IllegalArgumentException("获取数据库实例失败：" + dbname);
        }
    }

    public MongoDatabase getMongoDatabase() {
        return mongoDatabase;
    }

    public List<String> getCollectionNames() {
        List<String> list = new ArrayList<>();
        MongoCursor<String> cursor = mongoDatabase.listCollectionNames().iterator();
        while (cursor.hasNext()) {
            list.add(cursor.next());
        }
        cursor.close();
        return list;
    }

    public MongoCollection<Document> getCollection(String collectionName) {
        MongoCollection<Document> coll = collections.get(collectionName);
        if (coll == null) {
            coll = mongoDatabase.getCollection(collectionName);
            collections.put(collectionName, coll);
        }
        return coll;
    }

    public List<Document> findAll(String collectionName) {
        return find(collectionName, null, null, 0, 0);
    }

    public List<Document> findAll(String collectionName, Bson orderBy) {
        return find(collectionName, null, orderBy, 0, 0);
    }

    public List<Document> findAll(String collectionName, Bson orderBy, int limit) {
        return find(collectionName, null, orderBy, limit, 0);
    }

    public List<Document> findAll(String collectionName, int limit) {
        return find(collectionName, null, null, limit, 0);
    }

    public List<Document> findAll(String collectionName, int limit, int skip) {
        return find(collectionName, null, null, limit, skip);
    }

    public List<Document> findAll(String collectionName, Bson orderBy, int limit, int skip) {
        return find(collectionName, null, orderBy, limit, skip);
    }

    private <T> String getCollectionName(Class<T> clazz) {
        if (classCollectionName.containsKey(clazz)) {
            return classCollectionName.get(clazz);
        }
        String collectionName = clazz.getSimpleName();
        Collection collection = clazz.getAnnotation(Collection.class);
        if (collection != null && collection.value() != null && !collection.value().isEmpty()) {
            collectionName = collection.value();
        }
        return classCollectionName.put(clazz, collectionName);
    }

    public <T> List<T> findAll(Class<T> clazz) {
        String collectionName = getCollectionName(clazz);
        return find(clazz, collectionName, null, null, 0, 0);
    }

    public <T> List<T> findAll(Class<T> clazz, String collectionName) {
        return find(clazz, collectionName, null, null, 0, 0);
    }

    public <T> List<T> findAll(Class<T> clazz, String collectionName, Bson orderBy) {
        return find(clazz, collectionName, null, orderBy, 0, 0);
    }

    public <T> List<T> findAll(Class<T> clazz, String collectionName, Bson orderBy, int limit) {
        return find(clazz, collectionName, null, orderBy, limit, 0);
    }

    public <T> List<T> findAll(Class<T> clazz, String collectionName, int limit) {
        return find(clazz, collectionName, null, null, limit, 0);
    }

    public <T> List<T> findAll(Class<T> clazz, String collectionName, int limit, int skip) {
        return find(clazz, collectionName, null, null, limit, skip);
    }

    public <T> List<T> findAll(Class<T> clazz, String collectionName, Bson orderBy, int limit, int skip) {
        return find(clazz, collectionName, null, orderBy, limit, skip);
    }

    public List<Document> find(String collectionName, Bson filter) {
        return find(collectionName, filter, null, 0, 0);
    }

    public List<Document> find(String collectionName, Bson filter, Bson orderBy) {
        return find(collectionName, filter, orderBy, 0, 0);
    }

    public List<Document> find(String collectionName, Bson filter, Bson orderBy, int limit) {
        return find(collectionName, filter, orderBy, 0, 0);
    }

    public List<Document> find(String collectionName, Bson filter, int limit) {
        return find(collectionName, filter, null, 0, 0);
    }

    public List<Document> find(String collectionName, Bson filter, int limit, int skip) {
        return find(collectionName, filter, null, 0, 0);
    }

    public List<Document> find(String collectionName, Bson filter, Bson orderBy, int limit, int skip) {
        FindIterable<Document> find = null;
        if (filter == null) {
            find = getCollection(collectionName).find();
        } else {
            find = getCollection(collectionName).find(filter);
        }
        if (orderBy != null) {
            find.sort(orderBy);
        }
        if (skip > 0) {
            find.skip(skip);
        }
        if (limit > 0) {
            find.limit(limit);
        }
        MongoCursor<Document> cursor = find.iterator();
        List<Document> list = new ArrayList<>();
        try {
            while (cursor.hasNext()) {
                list.add(cursor.next());
            }
        } finally {
            cursor.close();
        }
        return list;
    }

    public <T> List<T> find(Class<T> clazz, Bson filter) {
        String collectionName = getCollectionName(clazz);
        return find(clazz, collectionName, filter, null, 0, 0);
    }

    public <T> List<T> find(Class<T> clazz, Bson filter, Bson orderBy) {
        String collectionName = getCollectionName(clazz);
        return find(clazz, collectionName, filter, orderBy, 0, 0);
    }

    public <T> List<T> find(Class<T> clazz, Bson filter, Bson orderBy, int limit) {
        String collectionName = getCollectionName(clazz);
        return find(clazz, collectionName, filter, orderBy, limit, 0);
    }

    public <T> List<T> find(Class<T> clazz, Bson filter, int limit) {
        String collectionName = getCollectionName(clazz);
        return find(clazz, collectionName, filter, null, limit, 0);
    }

    public <T> List<T> find(Class<T> clazz, Bson filter, int limit, int skip) {
        String collectionName = getCollectionName(clazz);
        return find(clazz, collectionName, filter, null, limit, skip);
    }

    public <T> List<T> find(Class<T> clazz, String collectionName, Bson filter) {
        return find(clazz, collectionName, filter, null, 0, 0);
    }

    public <T> List<T> find(Class<T> clazz, String collectionName, Bson filter, Bson orderBy) {
        return find(clazz, collectionName, filter, orderBy, 0, 0);
    }

    public <T> List<T> find(Class<T> clazz, String collectionName, Bson filter, Bson orderBy, int limit) {
        return find(clazz, collectionName, filter, orderBy, limit, 0);
    }

    public <T> List<T> find(Class<T> clazz, String collectionName, Bson filter, int limit) {
        return find(clazz, collectionName, filter, null, limit, 0);
    }

    public <T> List<T> find(Class<T> clazz, String collectionName, Bson filter, int limit, int skip) {
        return find(clazz, collectionName, filter, null, limit, skip);
    }

    public <T> List<T> find(Class<T> clazz, String collectionName, Bson filter, Bson orderBy, int limit, int skip) {
        FindIterable<Document> find = null;
        if (filter == null) {
            find = getCollection(collectionName).find();
        } else {
            find = getCollection(collectionName).find(filter);
        }
        if (orderBy != null) {
            find.sort(orderBy);
        }
        if (skip > 0) {
            find.skip(skip);
        }
        if (limit > 0) {
            find.limit(limit);
        }
        MongoCursor<Document> cursor = find.iterator();
        List<T> list = new ArrayList<T>();
        try {
            while (cursor.hasNext()) {
                Document doc = cursor.next();
                ObjectId id = doc.getObjectId(ID);
                if (id != null) {
                    doc.put(ID, id.toHexString());
                }
                String json = doc.toJson();
                list.add(SerializeHelper.deserialize(json, clazz));
            }
        } finally {
            cursor.close();
        }
        return list;
    }

    public List<Document> distinct(String collectionName, String fieldName, Bson filter) {
        DistinctIterable<Document> find = null;
        if (filter == null) {
            find = getCollection(collectionName).distinct(fieldName, Document.class);
        } else {
            find = getCollection(collectionName).distinct(fieldName, filter, Document.class);
        }
        MongoCursor<Document> cursor = find.iterator();
        List<Document> list = new ArrayList<>();
        try {
            while (cursor.hasNext()) {
                list.add(cursor.next());
            }
        } finally {
            cursor.close();
        }
        return list;
    }

    public Document findOne(String collectionName, Bson filter) {
        MongoCursor<Document> cursor = getCollection(collectionName).find(filter).iterator();
        try {
            if (cursor.hasNext()) {
                return cursor.next();
            }
        } finally {
            cursor.close();
        }
        return null;
    }

    public <T> T findOne(Class<T> clazz, Bson filter) {
        String collectionName = getCollectionName(clazz);
        return findOne(clazz, collectionName, filter);
    }

    public <T> T findOne(Class<T> clazz, String collectionName, Bson filter) {
        MongoCursor<Document> cursor = getCollection(collectionName).find(filter).iterator();
        try {
            if (cursor.hasNext()) {
                Document doc = cursor.next();
                ObjectId id = doc.getObjectId(ID);
                if (id != null) {
                    doc.put(ID, id.toHexString());
                }
                String json = doc.toJson();
                return SerializeHelper.deserialize(json, clazz);
            }
        } finally {
            cursor.close();
        }
        return null;
    }

    public <T> T findById(Class<T> clazz, UUID id) {
        String collectionName = getCollectionName(clazz);
        return findById(clazz, collectionName, id);
    }

    public <T> T findById(Class<T> clazz, String collectionName, UUID id) {
        Bson filter = Filters.eq(ID, id);
        MongoCursor<Document> cursor = getCollection(collectionName).find(filter).iterator();
        try {
            if (cursor.hasNext()) {
                Document doc = cursor.next();
                ObjectId _id = doc.getObjectId(ID);
                if (_id != null) {
                    doc.put(ID, _id.toHexString());
                }
                String json = doc.toJson();
                return SerializeHelper.deserialize(json, clazz);
            }
        } finally {
            cursor.close();
        }
        return null;
    }

    public Document findById(String collectionName, UUID id) {
        Bson filter = Filters.eq(ID, id);
        MongoCursor<Document> cursor = getCollection(collectionName).find(filter).iterator();
        try {
            if (cursor.hasNext()) {
                return cursor.next();
            }
        } finally {
            cursor.close();
        }
        return null;
    }

    public <T> long count(Class<T> clazz) {
        String collectionName = getCollectionName(clazz);
        return count(collectionName);
    }

    public long count(String collectionName) {
        return getCollection(collectionName).countDocuments();
    }

    public <T> long count(Class<T> clazz, Bson filter) {
        String collectionName = getCollectionName(clazz);
        return count(collectionName, filter);
    }

    public long count(String collectionName, Bson filter) {
        if (filter == null) {
            return getCollection(collectionName).countDocuments();
        }
        return getCollection(collectionName).countDocuments(filter);
    }

    public List<Document> count(String collectionName, String[] groupBy) {
        return count(collectionName, groupBy, null, 0);
    }

    public List<Document> count(String collectionName, String[] groupBy, Bson filter) {
        return count(collectionName, groupBy, filter, 0);
    }

    public List<Document> count(String collectionName, String[] groupBy, Bson filter, int limit) {
        StringBuilder mapFunction = new StringBuilder("function(){emit(");
        int len = groupBy.length;
        if (len == 1) {
            mapFunction.append("this.").append(groupBy[0]);
        } else {
            mapFunction.append("{");
            for (int i = 0; i < len; i++) {
                if (i > 0) {
                    mapFunction.append(",");
                }
                mapFunction.append(groupBy[i]).append(":this.").append(groupBy[i]);
            }
            mapFunction.append("}");
        }
        mapFunction.append(",1");
        mapFunction.append(");}");
        StringBuilder reduceFunction = new StringBuilder("function(key, values){");
        reduceFunction.append("var total = 0;");
        reduceFunction.append("values.forEach(function(val){total += val;});");
        reduceFunction.append("return total;");
        reduceFunction.append("}");
        MapReduceIterable<Document> find = getCollection(collectionName).mapReduce(mapFunction.toString(), reduceFunction.toString());
        if (filter != null) {
            find.filter(filter);
        }
        if (limit > 0) {
            find.limit(limit);
        }
        find.jsMode(true);
        MongoCursor<Document> cursor = find.iterator();
        List<Document> list = new ArrayList<Document>();
        try {
            while (cursor.hasNext()) {
                Document doc = cursor.next();
                if (len == 1) {
                    doc.put(groupBy[0], doc.get("_id"));
                } else {
                    doc.putAll((Document) doc.get("_id"));
                }
                doc.remove("_id");

				Object val = doc.get("value");
				if (val instanceof List) {
                    val = ((List) val).get(0);
				}
                long count = 0;
                if (val instanceof Number) {
                    count = ((Number)val).longValue();
                } else {
                    LOG.warn("{} is not a number!!! doc={}", val, doc);
                }
                doc.remove("value");
                doc.put("count", count);
                list.add(doc);
            }
        } finally {
            cursor.close();
        }
        return list;
    }

    public List<Document> mapReduce(String collectionName, String mapFunction, String reduceFunction) {
        return mapReduce(collectionName, mapFunction, reduceFunction, null, null, null, 0);
    }

    public List<Document> mapReduce(String collectionName, String mapFunction, String reduceFunction, Bson filter) {
        return mapReduce(collectionName, mapFunction, reduceFunction, null, filter, null, 0);
    }

    public List<Document> mapReduce(String collectionName, String mapFunction, String reduceFunction, Bson filter, Bson orderBy) {
        return mapReduce(collectionName, mapFunction, reduceFunction, null, filter, orderBy, 0);
    }

    public List<Document> mapReduce(String collectionName, String mapFunction, String reduceFunction, Bson filter, int limit) {
        return mapReduce(collectionName, mapFunction, reduceFunction, null, filter, null, limit);
    }

    public List<Document> mapReduce(String collectionName, String mapFunction, String reduceFunction, String finalizeFunction) {
        return mapReduce(collectionName, mapFunction, reduceFunction, finalizeFunction, null, null, 0);
    }

    public List<Document> mapReduce(String collectionName, String mapFunction, String reduceFunction, String finalizeFunction, Bson filter) {
        return mapReduce(collectionName, mapFunction, reduceFunction, finalizeFunction, filter, null, 0);
    }

    public List<Document> mapReduce(String collectionName, String mapFunction, String reduceFunction, String finalizeFunction, Bson filter, Bson orderBy) {
        return mapReduce(collectionName, mapFunction, reduceFunction, finalizeFunction, filter, orderBy, 0);
    }

    public List<Document> mapReduce(String collectionName, String mapFunction, String reduceFunction, String finalizeFunction, Bson filter, int limit) {
        return mapReduce(Document.class, collectionName, mapFunction, reduceFunction, finalizeFunction, filter, null, limit);
    }

    public List<Document> mapReduce(String collectionName, String mapFunction, String reduceFunction, String finalizeFunction, Bson filter, Bson orderBy, int
            limit) {
        return mapReduce(Document.class, collectionName, mapFunction, reduceFunction, finalizeFunction, filter, orderBy, limit);
    }

    public <T> List<T> mapReduce(Class<T> resultClass, String collectionName, String mapFunction, String reduceFunction, String finalizeFunction, Bson
            filter, int limit) {
        return mapReduce(resultClass, collectionName, mapFunction, reduceFunction, finalizeFunction, filter, null, limit);
    }

    public <T> List<T> mapReduce(Class<T> resultClass, String collectionName, String mapFunction, String reduceFunction, String finalizeFunction, Bson
            filter, Bson orderBy, int limit) {
        MapReduceIterable<T> find = getCollection(collectionName).mapReduce(mapFunction, reduceFunction, resultClass);
        if (filter != null) {
            find.filter(filter);
        }
        if (finalizeFunction != null && !finalizeFunction.isEmpty()) {
            find.finalizeFunction(finalizeFunction);
        }
        if (orderBy != null) {
            find.sort(orderBy);
        }
        if (limit > 0) {
            find.limit(limit);
        }
        find.jsMode(true);
        MongoCursor<T> cursor = find.iterator();
        List<T> list = new ArrayList<T>();
        try {
            while (cursor.hasNext()) {
                list.add(cursor.next());
            }
        } finally {
            cursor.close();
        }
        return list;
    }

    public void save(Object entity) {
        Class<?> clazz = entity.getClass();
        String collectionName = getCollectionName(clazz);
        save(entity, collectionName);
    }

    public void save(Object entity, String collectionName) {
        if (entity instanceof Document) {
            getCollection(collectionName).insertOne((Document) entity);
        } else if (entity instanceof Map) {
            getCollection(collectionName).insertOne(new Document((Map<String, Object>) entity));
        } else {
            String json = toJson(entity);
            getCollection(collectionName).insertOne(Document.parse(json));
        }
    }

    public void saveAll(List<?> list) {
        Class<?> clazz = list.get(0).getClass();
        String collectionName = getCollectionName(clazz);
        saveAll(list, collectionName);
    }

    public void saveAll(List<?> list, String collectionName) {
        List<Document> docList = new ArrayList<>();
        for (Object obj : list) {
            if (obj instanceof Document) {
                docList.add((Document) obj);
            } else if (obj instanceof Map) {
                docList.add(new Document((Map<String, Object>) obj));
            } else {
                String json = toJson(obj);
                docList.add(Document.parse(json));
            }
        }
        getCollection(collectionName).insertMany(docList);
    }

    public <T> boolean update(T entity) {
        String collectionName = getCollectionName(entity.getClass());
        return update(entity, collectionName);
    }

    public <T> boolean update(T entity, String collectionName) {
        UUID id = null;
        if (entity instanceof MongoEntity mongoEntity) {

        }
        return updateById(entity, collectionName, id);
    }


    public <T> long update(T entity, Bson filter) {
        String collectionName = getCollectionName(entity.getClass());
        return update(entity, collectionName, filter, false);
    }

    public <T> long update(T entity, Bson filter, boolean mutil) {
        String collectionName = getCollectionName(entity.getClass());
        return update(entity, collectionName, filter, mutil);
    }

    public <T> long update(T entity, String collectionName, Bson filter, boolean mutil) {
        String json = toJson(entity);
        BasicDBObject update = BasicDBObject.parse(json);
        return update(collectionName, filter, new BasicDBObject("$set", update), mutil);
    }

    public long update(String collectionName, Bson filter, UpdateEntity updateEntity) {
        return update(collectionName, filter, updateEntity, false);
    }

    public long update(String collectionName, Bson filter, UpdateEntity updateEntity, boolean mutil) {
        BasicDBObject _update = new BasicDBObject();
        if (!updateEntity.inc().isEmpty()) {
            _update.append("$inc", updateEntity.inc());
        }
        if (!updateEntity.set().isEmpty()) {
            _update.append("$set", updateEntity.set());
        }
        if (!updateEntity.unset().isEmpty()) {
            _update.append("$unset", updateEntity.unset());
        }
        return update(collectionName, filter, _update, mutil);
    }

    public long update(String collectionName, Bson filter, Bson update, boolean mutil) {
        UpdateResult result = null;
        if (mutil) {
            result = getCollection(collectionName).updateMany(filter, update);
        } else {
            result = getCollection(collectionName).updateOne(filter, update);
        }
        if (result.wasAcknowledged()) {
            return result.getModifiedCount();
        }
        return -1;
    }

    public <T> boolean updateById(T entity, UUID id) {
        String collectionName = getCollectionName(entity.getClass());
        return updateById(entity, collectionName, id);
    }

    public <T> boolean updateById(T entity, String collectionName, UUID id) {
        if (id == null) {
            throw new IllegalArgumentException("id不能为空");
        }
        String json = toJson(entity);
        Bson filter = Filters.eq(ID, id);
        BasicDBObject update = BasicDBObject.parse(json);
        UpdateResult result = getCollection(collectionName).updateOne(filter, new BasicDBObject("$set", update));
        if (result.wasAcknowledged()) {
            if (result.getModifiedCount() > 0) {
                return true;
            }
        } else {
            return true;
        }
        return false;
    }

    public boolean updateById(String collectionName, UpdateEntity updateEntity, UUID id) {
        BasicDBObject _update = new BasicDBObject();
        if (!updateEntity.inc().isEmpty()) {
            _update.append("$inc", updateEntity.inc());
        }
        if (!updateEntity.set().isEmpty()) {
            _update.append("$set", updateEntity.set());
        }
        if (!updateEntity.unset().isEmpty()) {
            _update.append("$unset", updateEntity.unset());
        }
        return updateById(collectionName, _update, id);
    }

    public boolean updateById(String collectionName, Bson update, UUID id) {
        if (id == null) {
            throw new IllegalArgumentException("id不能为空");
        }
        Bson filter = Filters.eq(ID, id);
        UpdateResult result = getCollection(collectionName).updateOne(filter, update);
        if (result.wasAcknowledged()) {
            return (result.getModifiedCount() > 0);
        }
        return true;
    }

    public boolean deleteOne(String collectionName, Bson filter) {
        DeleteResult result = getCollection(collectionName).deleteOne(filter);
        if (result.wasAcknowledged()) {
            return (result.getDeletedCount() > 0);
        }
        return true;
    }

    public long deleteAll(String collectionName, Bson filter) {
        DeleteResult result = getCollection(collectionName).deleteMany(filter);
        if (result.wasAcknowledged()) {
            return result.getDeletedCount();
        }
        return 0;
    }

    public boolean deleteById(String collectionName, UUID id) {
        return deleteOne(collectionName, Filters.eq(ID, id));
    }

    public void drop(String collectionName) {
        getCollection(collectionName).drop();
    }

    public void createIndex(String collectionName, Bson keys) {
        getCollection(collectionName).createIndex(keys);
    }

    public void dropIndexes(String collectionName) {
        getCollection(collectionName).dropIndexes();
    }

    public void dropIndex(String collectionName, String indexName) {
        getCollection(collectionName).dropIndex(indexName);
    }

    protected String toJson(Object obj) {
        return SerializeHelper.serialize(obj);
    }
}