package com.io.yutian.elementoriginlib.scanner;

import com.io.yutian.elementoriginlib.logger.Logger;
import com.io.yutian.elementoriginlib.shadowloader.BytecodeRewriter;
import com.io.yutian.elementoriginlib.shadowloader.ShadowClassLoader;
import javassist.ClassPool;
import javassist.CtClass;

import java.io.*;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

public class ClassScanner {

    private static final Logger LOGGER = Logger.getLogger(ClassScanner.class);
    private static final int THREAD_COUNT = Runtime.getRuntime().availableProcessors();
    public static final Set<String> DEFAULT_PACKAGES = new HashSet<>(Arrays.asList("com.io.yutian"));
    public static final Set<String> DEFAULT_EXCLUDE_PACKAGES = new HashSet<>(Arrays.asList("com.io.yutian.elementoriginlib"));

    private Set<String> packageNames;
    private Set<String> excludePackageNames;

    private ClassLoaderProcessor classLoaderProcessor;
    private ClassLoader classLoader;

    private Set<Class<?>> scannedClasses = ConcurrentHashMap.newKeySet();
    private Map<String, Object> scannedClassInstances = new ConcurrentHashMap<>();

    public ClassScanner(Set<String> packageNames, Set<String> excludePackageNames, ClassLoader classLoader, ClassLoaderProcessor classLoaderProcessor) {
        this.packageNames = packageNames;
        this.excludePackageNames = excludePackageNames;
        this.classLoader = classLoader;
        this.classLoaderProcessor = classLoaderProcessor;
    }

    public void scan(File pluginsFolder) {
        File[] jars = pluginsFolder.listFiles((dir, name) -> name.endsWith(".jar"));
        if (jars == null) return;
        ExecutorService executor = Executors.newFixedThreadPool(THREAD_COUNT);

        for (File jar : jars) {
            executor.submit(() -> {
                try {
                    scanJar(jar);
                } catch (Exception e) {
                    LOGGER.error(e.getMessage(), e);
                }
            });
        }
        executor.shutdown();
        try {
            executor.awaitTermination(60, TimeUnit.SECONDS);
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
        }
    }

    private void scanJar(File jarFile) throws IOException {
        try (JarFile jar = new JarFile(jarFile)) {
            Enumeration<JarEntry> entries = jar.entries();
            URLClassLoader urlClassLoader = new URLClassLoader(new URL[]{jarFile.toURI().toURL()}, classLoader);
            ShadowClassLoader shadowLoader = new ShadowClassLoader(classLoader);

            while (entries.hasMoreElements()) {
                JarEntry entry = entries.nextElement();
                if (entry.getName().endsWith(".class")) {
                    String className = entry.getName().replace("/", ".").replace(".class", "");
                    if (excludePackageNames.stream().anyMatch(className::startsWith)) {
                        continue;
                    }
                    if (packageNames.stream().noneMatch(className::startsWith)) {
                        continue;
                    }
                    try (InputStream is = jar.getInputStream(entry)) {
                        byte[] originalBytes = is.readAllBytes();
                        Class<?> clazz = urlClassLoader.loadClass(className);
                        if (classLoaderProcessor.isMatch(clazz, originalBytes)) {
                            String newName = "lib.shadow." + clazz.getName();
                            byte[] newBytes = BytecodeRewriter.rewriteClass(originalBytes, className, newName);
                            Class<?> newClass = shadowLoader.define(newName, newBytes);
                            Object instance = newClass.newInstance();
                            classLoaderProcessor.process(newClass, instance);
                            scannedClasses.add(clazz);
                            scannedClassInstances.put(clazz.getName(), instance);
                        }
                    } catch (Throwable ignored) {
                        LOGGER.error(ignored.getMessage(), ignored);
                    }
                }
            }
        }
    }

    public static Map<String, Map<String, Object>> getClassAnnotations(byte[] classBytes) {
        Map<String, Map<String, Object>> annotations = new HashMap<>();
        try (DataInputStream in = new DataInputStream(new ByteArrayInputStream(classBytes))) {
            in.skipBytes(8); // magic, minor, major
            int cpCount = in.readUnsignedShort();

            Map<Integer, String> utf8 = new HashMap<>();
            Map<Integer, Integer> classInfo = new HashMap<>();

            for (int i = 1; i < cpCount; i++) {
                int tag = in.readUnsignedByte();
                switch (tag) {
                    case 1 -> utf8.put(i, in.readUTF());
                    case 3, 4 -> in.skipBytes(4);
                    case 5, 6 -> { in.skipBytes(8); i++; }
                    case 7 -> classInfo.put(i, in.readUnsignedShort());
                    case 8 -> in.readUnsignedShort();
                    case 9, 10, 11, 12, 18 -> in.skipBytes(4);
                    case 15 -> in.skipBytes(3);
                    case 16 -> in.skipBytes(2);
                    case 17 -> in.skipBytes(4);
                    default -> throw new IOException("Bad constant pool tag: " + tag);
                }
            }

            in.skipBytes(2 + 2 + 2); // access_flags, this_class, super_class

            int interfacesCount = in.readUnsignedShort();
            in.skipBytes(2 * interfacesCount);

            int fieldsCount = in.readUnsignedShort();
            skipAttributes(in, fieldsCount);

            int methodsCount = in.readUnsignedShort();
            skipAttributes(in, methodsCount);

            int attributesCount = in.readUnsignedShort();
            for (int i = 0; i < attributesCount; i++) {
                int nameIndex = in.readUnsignedShort();
                String attrName = utf8.get(nameIndex);
                int length = in.readInt();
                if ("RuntimeVisibleAnnotations".equals(attrName)) {
                    int numAnnotations = in.readUnsignedShort();
                    for (int j = 0; j < numAnnotations; j++) {
                        int typeIndex = in.readUnsignedShort();
                        String desc = utf8.get(typeIndex);
                        if (desc == null) continue;
                        String annotationClass = desc.substring(1, desc.length() - 1).replace("/", ".");
                        Map<String, Object> values = readAnnotationValues(in, utf8);
                        annotations.put(annotationClass, values);
                    }
                } else {
                    in.skipBytes(length);
                }
            }
        } catch (IOException ignored) {
        }
        return annotations;
    }

    private static void skipAttributes(DataInputStream in, int count) throws IOException {
        for (int i = 0; i < count; i++) {
            in.skipBytes(6); // access_flags, name_index, descriptor_index
            int attributesCount = in.readUnsignedShort();
            for (int j = 0; j < attributesCount; j++) {
                in.skipBytes(2); // attribute_name_index
                int length = in.readInt();
                in.skipBytes(length);
            }
        }
    }

    private static Map<String, Object> readAnnotationValues(DataInputStream in, Map<Integer, String> utf8) throws IOException {
        Map<String, Object> map = new HashMap<>();
        int numPairs = in.readUnsignedShort();
        for (int i = 0; i < numPairs; i++) {
            int elementNameIndex = in.readUnsignedShort();
            String elementName = utf8.get(elementNameIndex);
            Object value = readElementValue(in, utf8);
            map.put(elementName, value);
        }
        return map;
    }

    private static Object readElementValue(DataInputStream in, Map<Integer, String> utf8) throws IOException {
        char tag = (char) in.readUnsignedByte();
        return switch (tag) {
            case 'B', 'C', 'D', 'F', 'I', 'J', 'S', 'Z', 's' -> {
                int constIndex = in.readUnsignedShort();
                yield utf8.get(constIndex);
            }
            case 'e' -> {
                in.skipBytes(4);
                yield "<Enum>";
            }
            case 'c' -> {
                in.skipBytes(2);
                yield "<Class>";
            }
            case '@' -> {
                skipAnnotation(in, utf8);
                yield "<Annotation>";
            }
            case '[' -> {
                int numValues = in.readUnsignedShort();
                List<Object> list = new ArrayList<>();
                for (int i = 0; i < numValues; i++) {
                    list.add(readElementValue(in, utf8));
                }
                yield list;
            }
            default -> null;
        };
    }

    private static void skipAnnotation(DataInputStream in, Map<Integer, String> utf8) throws IOException {
        in.skipBytes(2);
        int numPairs = in.readUnsignedShort();
        for (int i = 0; i < numPairs; i++) {
            in.skipBytes(2);
            readElementValue(in, utf8);
        }
    }

    public static String getSuperClassName(byte[] classBytes) {
        try (DataInputStream in = new DataInputStream(new ByteArrayInputStream(classBytes))) {
            in.skipBytes(8); // magic + minor + major
            int constantPoolCount = in.readUnsignedShort();

            Map<Integer, String> utf8Constants = new HashMap<>();
            Map<Integer, Integer> classInfo = new HashMap<>();

            for (int i = 1; i < constantPoolCount; i++) {
                int tag = in.readUnsignedByte();
                switch (tag) {
                    case 1 -> utf8Constants.put(i, in.readUTF()); // Utf8
                    case 3, 4 -> in.skipBytes(4);
                    case 5, 6 -> { in.skipBytes(8); i++; }
                    case 7 -> classInfo.put(i, in.readUnsignedShort());
                    case 8 -> in.readUnsignedShort();
                    case 9, 10, 11, 12, 18 -> in.skipBytes(4);
                    case 15 -> in.skipBytes(3);
                    case 16 -> in.skipBytes(2);
                    case 17 -> in.skipBytes(4);
                    default -> throw new IOException("Bad constant pool tag: " + tag);
                }
            }

            in.skipBytes(2); // access_flags
            in.skipBytes(2); // this_class
            int superClassIndex = in.readUnsignedShort();

            Integer nameIndex = classInfo.get(superClassIndex);
            if (nameIndex != null) {
                return utf8Constants.get(nameIndex).replace("/", ".");
            }
        } catch (IOException e) {
            return null;
        }
        return null;
    }

    public static Set<String> getInterfaceNames(byte[] classBytes) {
        Set<String> interfaces = new HashSet<>();
        try (DataInputStream in = new DataInputStream(new ByteArrayInputStream(classBytes))) {
            in.skipBytes(8); // magic + minor + major
            int constantPoolCount = in.readUnsignedShort();

            Map<Integer, String> utf8Constants = new HashMap<>();
            Map<Integer, Integer> classInfo = new HashMap<>();

            for (int i = 1; i < constantPoolCount; i++) {
                int tag = in.readUnsignedByte();
                switch (tag) {
                    case 1 -> utf8Constants.put(i, in.readUTF()); // Utf8
                    case 3, 4 -> in.skipBytes(4);                 // int, float
                    case 5, 6 -> { in.skipBytes(8); i++; }        // long, double
                    case 7 -> classInfo.put(i, in.readUnsignedShort()); // Class -> name_index
                    case 8 -> in.readUnsignedShort();            // String
                    case 9, 10, 11, 12, 18 -> in.skipBytes(4);    // Field/Method/InterfaceMethod/NameAndType/InvokeDynamic
                    case 15 -> in.skipBytes(3);                  // MethodHandle
                    case 16 -> in.skipBytes(2);                  // MethodType
                    case 17 -> in.skipBytes(4);                  // Dynamic
                    default -> throw new IOException("Bad constant pool tag: " + tag);
                }
            }

            in.skipBytes(2); // access flags
            in.skipBytes(2); // this class
            in.skipBytes(2); // super class

            int interfacesCount = in.readUnsignedShort();
            for (int i = 0; i < interfacesCount; i++) {
                int index = in.readUnsignedShort();
                Integer nameIndex = classInfo.get(index);
                if (nameIndex != null) {
                    String iface = utf8Constants.get(nameIndex);
                    if (iface != null) {
                        interfaces.add(iface.replace("/", "."));
                    }
                }
            }
            return interfaces;
        } catch (IOException e) {
            return interfaces;
        }
    }

    public Set<Class<?>> getScannedClasses() {
        return Collections.unmodifiableSet(scannedClasses);
    }

    public Map<String, Object> getScannedClassInstances() {
        return Collections.unmodifiableMap(scannedClassInstances);
    }

}
