package com.io.yutian.elementoriginlib.util;

import com.google.common.collect.ImmutableMap;
import net.md_5.bungee.api.ChatColor;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.ThreadSafe;
import java.awt.*;
import java.util.*;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;

public class ColorUtil {

    public static final List<String> SPECIAL_COLORS = Arrays.asList("&l", "&n", "&o", "&k", "&m");

    private static final Map<Color, ChatColor> COLORS = ImmutableMap.<Color, ChatColor>builder()
            .put(new Color(0), ChatColor.getByChar('0'))
            .put(new Color(170), ChatColor.getByChar('1'))
            .put(new Color(43520), ChatColor.getByChar('2'))
            .put(new Color(43690), ChatColor.getByChar('3'))
            .put(new Color(11141120), ChatColor.getByChar('4'))
            .put(new Color(11141290), ChatColor.getByChar('5'))
            .put(new Color(16755200), ChatColor.getByChar('6'))
            .put(new Color(11184810), ChatColor.getByChar('7'))
            .put(new Color(5592405), ChatColor.getByChar('8'))
            .put(new Color(5592575), ChatColor.getByChar('9'))
            .put(new Color(5635925), ChatColor.getByChar('a'))
            .put(new Color(5636095), ChatColor.getByChar('b'))
            .put(new Color(16733525), ChatColor.getByChar('c'))
            .put(new Color(16733695), ChatColor.getByChar('d'))
            .put(new Color(16777045), ChatColor.getByChar('e'))
            .put(new Color(16777215), ChatColor.getByChar('f'))
            .build();

    private static final List<Pattern> PATTERNS = Arrays.asList(new GradientPattern(), new SolidPattern(), new RainbowPattern());

    private static final LruCache LRU_CACHE = new LruCache(512);

    @Nonnull
    public static String process(@Nonnull String string) {
        String result = LRU_CACHE.getResult(string);
        if (result != null) {
            return result;
        }
        String input = string;
        for (Pattern pattern : PATTERNS) {
            string = pattern.process(string);
        }
        string = ChatColor.translateAlternateColorCodes('&', string);
        LRU_CACHE.put(input, string);
        return string;
    }

    @Nonnull
    public static List<String> process(@Nonnull List<String> strings) {
        strings.replaceAll(ColorUtil::process);
        return strings;
    }

    @Nonnull
    public static String color(@Nonnull String string, @Nonnull Color color) {
        return ChatColor.of(color) + string;
    }

    @Nonnull
    public static String color(@Nonnull String string, @Nonnull Color start, @Nonnull Color end) {
        StringBuilder specialColors = new StringBuilder();
        for (String color : SPECIAL_COLORS) {
            if (string.contains(color)) {
                specialColors.append(color);
                string = string.replace(color, "");
            }
        }
        StringBuilder stringBuilder = new StringBuilder();
        ChatColor[] colors = createGradient(start, end, string.length());
        String[] characters = string.split("");
        for (int i = 0; i < string.length(); i++) {
            stringBuilder.append(colors[i]).append(specialColors).append(characters[i]);
        }
        return stringBuilder.toString();
    }

    @Nonnull
    public static String rainbow(@Nonnull String string, float saturation) {
        StringBuilder specialColors = new StringBuilder();
        for (String color : SPECIAL_COLORS) {
            if (string.contains(color)) {
                specialColors.append(color);
                string = string.replace(color, "");
            }
        }
        StringBuilder stringBuilder = new StringBuilder();
        ChatColor[] colors = createRainbow(string.length(), saturation);
        String[] characters = string.split("");
        for (int i = 0; i < string.length(); i++) {
            stringBuilder.append(colors[i]).append(specialColors).append(characters[i]);
        }
        return stringBuilder.toString();
    }

    @Nonnull
    public static ChatColor getColor(@Nonnull String string) {
        return  ChatColor.of(new Color(Integer.parseInt(string, 16)));
    }

    @Nonnull
    public static String stripColorFormatting(@Nonnull String string) {
        return string.replaceAll("[&§][a-f0-9lnokm]|<[/]?\\w{5,8}(:[0-9A-F]{6})?>", "");
    }

    @Nonnull
    private static ChatColor[] createRainbow(int step, float saturation) {
        ChatColor[] colors = new ChatColor[step];
        double colorStep = (1.00 / step);
        for (int i = 0; i < step; i++) {
            Color color = Color.getHSBColor((float) (colorStep * i), saturation, saturation);
            colors[i] = ChatColor.of(color);
        }
        return colors;
    }

    @Nonnull
    private static ChatColor[] createGradient(@Nonnull Color start, @Nonnull Color end, int step) {
        if (step <= 1) {
            return new ChatColor[]{ChatColor.WHITE, ChatColor.WHITE, ChatColor.WHITE};
        }

        ChatColor[] colors = new ChatColor[step];
        int stepR = Math.abs(start.getRed() - end.getRed()) / (step - 1);
        int stepG = Math.abs(start.getGreen() - end.getGreen()) / (step - 1);
        int stepB = Math.abs(start.getBlue() - end.getBlue()) / (step - 1);
        int[] direction = new int[]{
                start.getRed() < end.getRed() ? +1 : -1,
                start.getGreen() < end.getGreen() ? +1 : -1,
                start.getBlue() < end.getBlue() ? +1 : -1
        };

        for (int i = 0; i < step; i++) {
            Color color = new Color(start.getRed() + ((stepR * i) * direction[0]), start.getGreen() + ((stepG * i) * direction[1]), start.getBlue() + ((stepB * i) * direction[2]));
            colors[i] = ChatColor.of(color);
        }
        return colors;
    }


    @Nonnull
    private static ChatColor getClosestColor(Color color) {
        Color nearestColor = null;
        double nearestDistance = Integer.MAX_VALUE;

        for (Color constantColor : COLORS.keySet()) {
            double distance = Math.pow(color.getRed() - constantColor.getRed(), 2) + Math.pow(color.getGreen() - constantColor.getGreen(), 2) + Math.pow(color.getBlue() - constantColor.getBlue(), 2);
            if (nearestDistance > distance) {
                nearestColor = constantColor;
                nearestDistance = distance;
            }
        }
        return COLORS.get(nearestColor);
    }

    @ThreadSafe
    private static class LruCache {

        private final Deque<String> deque = new LinkedList<>();
        private final Map<String, LruElement> map = new ConcurrentHashMap<>();

        private final int maxSize;

        public LruCache(int maxSize) {
            this.maxSize = maxSize;
        }

        public String getResult(String input) {
            if (input != null && map.containsKey(input)) {
                LruElement curr = map.get(input);
                synchronized (deque) {
                    deque.remove(input);
                    deque.addFirst(input);
                }
                return curr.getResult();
            }

            return null;
        }

        public void put(String input, String result) {
            if (input == null || result == null) {
                return;
            }
            synchronized (deque) {
                if (map.containsKey(input)) {
                    deque.remove(input);
                } else {
                    int size = deque.size();
                    if (size == maxSize && size > 0) {
                        String temp = deque.removeLast();
                        map.remove(temp);
                    }
                }
                LruElement newObj = new LruElement(input, result);
                deque.addFirst(input);
                map.put(input, newObj);
            }
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            LruCache lruCache = (LruCache) o;
            return maxSize == lruCache.maxSize && deque.equals(lruCache.deque) && map.equals(lruCache.map);
        }

        @Override
        public int hashCode() {
            int result = deque.hashCode();
            result = 31 * result + map.hashCode();
            result = 31 * result + maxSize;
            return result;
        }
    }

    private static class LruElement {

        private final String input;
        private final String result;

        public LruElement(String input, String result) {
            this.input = input;
            this.result = result;
        }

        public String getInput() {
            return input;
        }

        public String getResult() {
            return result;
        }
    }

    private static interface Pattern {

        String process(String string);

    }

    private static class GradientPattern implements Pattern {

        private static final java.util.regex.Pattern PATTERN = java.util.regex.Pattern.compile(
                "[<{]#([A-Fa-f0-9]{6})[}>](((?![<{]#[A-Fa-f0-9]{6}[}>]).)*)[<{]/#([A-Fa-f0-9]{6})[}>]"
        );

        public String process(String string) {
            Matcher matcher = PATTERN.matcher(string);
            while (matcher.find()) {
                String start = matcher.group(1);
                String content = matcher.group(2);
                String end = matcher.group(4);

                string = string.replace(
                        matcher.group(),
                        color(
                                content,
                                new Color(Integer.parseInt(start, 16)),
                                new Color(Integer.parseInt(end, 16))
                        )
                );
            }
            return string;
        }

    }

    private static class RainbowPattern implements Pattern {

        java.util.regex.Pattern pattern = java.util.regex.Pattern.compile("<RAINBOW([0-9]{1,3})>(.*?)</RAINBOW>");

        public String process(String string) {
            Matcher matcher = pattern.matcher(string);
            while (matcher.find()) {
                String saturation = matcher.group(1);
                String content = matcher.group(2);

                string = string.replace(matcher.group(), rainbow(content, Float.parseFloat(saturation)));
            }
            return string;
        }

    }

    private static class SolidPattern implements Pattern {

        public static final java.util.regex.Pattern PATTERN = java.util.regex.Pattern.compile("[<{]#([A-Fa-f0-9]{6})[}>]|[&]?#([A-Fa-f0-9]{6})");

        public String process(String string) {
            Matcher matcher = PATTERN.matcher(string);
            while (matcher.find()) {
                String color = matcher.group(1);
                if (color == null) {
                    color = matcher.group(2);
                }

                string = string.replace(matcher.group(), getColor(color) + "");
            }
            return string;
        }

    }

}
