package com.io.yutian.elementoriginlib.util;

import com.io.yutian.elementoriginlib.ElementOriginLib;
import com.io.yutian.elementoriginlib.logger.Logger;
import com.mojang.authlib.GameProfile;
import com.mojang.authlib.properties.Property;
import net.kyori.adventure.text.Component;
import org.bukkit.*;
import org.bukkit.attribute.Attribute;
import org.bukkit.attribute.AttributeModifier;
import org.bukkit.block.banner.Pattern;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.inventory.EquipmentSlot;
import org.bukkit.inventory.ItemFlag;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.*;
import org.bukkit.inventory.meta.trim.ArmorTrim;
import org.bukkit.potion.PotionEffect;
import org.jetbrains.annotations.Nullable;

import java.lang.reflect.Field;
import java.util.*;

public class ItemStackBuilder {

    private ItemStack itemStack;

    public ItemStackBuilder(Material material) {
        this.itemStack = new ItemStack(material);
    }

    public ItemStackBuilder(ItemStack item) {
        this.itemStack = item.clone();
    }

    public ItemStack build() {
        return itemStack;
    }

    public int getAmount() {
        return itemStack.getAmount();
    }

    public Material getType() {
        return itemStack.getType();
    }

    public List<Component> getLore() {
        return itemStack.getItemMeta().lore();
    }

    public int getCustomModelData() {
        return itemStack.getItemMeta().getCustomModelData();
    }

    public Map<Enchantment, Integer> getEnchants() {
        return itemStack.getItemMeta().getEnchants();
    }

    public Set<ItemFlag> getItemFlags() {
        return itemStack.getItemMeta().getItemFlags();
    }

    public ItemStackBuilder setItemStack(ItemStack item) {
        this.itemStack = item;
        return this;
    }

    public ItemStackBuilder setType(Material type) {
        this.itemStack.withType(type);
        return this;
    }

    public ItemStackBuilder setCustomModelData(int customModelData) {
        this.itemStack.editMeta((meta)-> meta.setCustomModelData(customModelData));
        return this;
    }

    public ItemStackBuilder setAmount(int amount) {
        this.itemStack.setAmount(amount);
        return this;
    }

    public ItemStackBuilder setDisplayName(String name) {
        this.itemStack.editMeta((meta)-> meta.displayName(Component.text(name)));
        return this;
    }

    public ItemStackBuilder setDisplayName(Component name) {
        this.itemStack.editMeta((meta)-> meta.displayName(name));
        return this;
    }

    public ItemStackBuilder setLore(List<String> loreList) {
        List<Component> loreComponents = new ArrayList<>();
        for (String lore : loreList) {
            loreComponents.add(Component.text(lore));
        }
        this.itemStack.editMeta((meta)-> meta.lore(loreComponents));
        return this;
    }

    public ItemStackBuilder setLore(String... lores) {
        return setLore(List.of(lores));
    }

    public ItemStackBuilder addLore(String lore) {
        List<Component> loreComponents = itemStack.getItemMeta().hasLore() ? itemStack.getItemMeta().lore() : new ArrayList<>();
        loreComponents.add(Component.text(lore));
        this.itemStack.editMeta((meta)-> meta.lore(loreComponents));
        return this;
    }

    public ItemStackBuilder addLore(String... lores) {
        List<Component> loreComponents = itemStack.getItemMeta().hasLore() ? itemStack.getItemMeta().lore() : new ArrayList<>();
        for (String lore : lores) {
            loreComponents.add(Component.text(lore));
        }
        this.itemStack.editMeta((meta)-> meta.lore(loreComponents));
        return this;
    }

    public ItemStackBuilder addLore(Component... lores) {
        List<Component> loreComponents = itemStack.getItemMeta().hasLore() ? itemStack.getItemMeta().lore() : new ArrayList<>();
        for (Component lore : lores) {
            loreComponents.add(lore);
        }
        this.itemStack.editMeta((meta)-> meta.lore(loreComponents));
        return this;
    }

    public ItemStackBuilder addLore(List<Component> lores) {
        List<Component> loreComponents = itemStack.getItemMeta().hasLore() ? itemStack.getItemMeta().lore() : new ArrayList<>();
        loreComponents.addAll(lores);
        this.itemStack.editMeta((meta)-> meta.lore(loreComponents));
        return this;
    }

    public ItemStackBuilder addAttributeModifier(Attribute attribute, AttributeModifier attributeModifier) {
        this.itemStack.editMeta((meta)-> meta.addAttributeModifier(attribute, attributeModifier));
        return this;
    }

    public ItemStackBuilder removeAttributeModifier(Attribute attribute) {
        this.itemStack.editMeta((meta)-> meta.removeAttributeModifier(attribute));
        return this;
    }

    public ItemStackBuilder removeAttributeModifier(EquipmentSlot equipmentSlot) {
        this.itemStack.editMeta((meta)-> meta.removeAttributeModifier(equipmentSlot));
        return this;
    }

    public ItemStackBuilder removeAttributeModifier(Attribute attribute, AttributeModifier attributeModifier) {
        this.itemStack.editMeta((meta)-> meta.removeAttributeModifier(attribute, attributeModifier));
        return this;
    }

    public ItemStackBuilder addEnchant(Enchantment ench, int level) {
        this.itemStack.editMeta((meta)-> meta.addEnchant(ench, level, true));
        return this;
    }

    public ItemStackBuilder removeEnchant(Enchantment ench) {
        this.itemStack.editMeta((meta)-> meta.removeEnchant(ench));
        return this;
    }

    public ItemStackBuilder setEnchants(Map<Enchantment, Integer> enchants) {
        this.itemStack.editMeta((meta)->{
            meta.removeEnchantments();
            for (Map.Entry<Enchantment, Integer> entry : enchants.entrySet()) {
                meta.addEnchant(entry.getKey(), entry.getValue(), true);
            }
        });
        return this;
    }

    public ItemStackBuilder setItemFlags(Set<ItemFlag> itemFlags) {
        this.itemStack.editMeta((meta)-> {
            meta.removeItemFlags(ItemFlag.values());
            for (ItemFlag itemFlag : itemFlags) {
                meta.addItemFlags(itemFlag);
            }
        });
        return this;
    }

    public ItemStackBuilder addItemFlags(ItemFlag[] itemFlags) {
        this.itemStack.editMeta((meta)-> {
            meta.removeItemFlags(ItemFlag.values());
            for (ItemFlag itemFlag : itemFlags) {
                meta.addItemFlags(itemFlag);
            }
        });
        return this;
    }

    public ItemStackBuilder addItemFlags(ItemFlag itemFlag) {
        this.itemStack.editMeta((meta)-> meta.addItemFlags(itemFlag));
        return this;
    }

    public ItemStackBuilder setUnbreakable(boolean unbreakable) {
        this.itemStack.editMeta((meta)-> meta.setUnbreakable(unbreakable));
        return this;
    }

    public ItemStackBuilder setHideTooltip(boolean hideTooltip) {
        this.itemStack.editMeta((meta)-> meta.setHideTooltip(hideTooltip));
        return this;
    }

    public ItemStackBuilder setFireResistant(boolean fireResistant) {
        this.itemStack.editMeta((meta)-> meta.setFireResistant(fireResistant));
        return this;
    }

    public ItemStackBuilder setMaxStackSize(int maxStackSize) {
        this.itemStack.editMeta((meta)-> meta.setCustomModelData(maxStackSize));
        return this;
    }

    public ItemStackBuilder setEnchantmentGlint(boolean enchantmentGlint) {
        this.itemStack.editMeta((meta)-> meta.setEnchantmentGlintOverride(enchantmentGlint));
        return this;
    }

    public ItemStackBuilder addPotionEffect(PotionEffect potionEffect) {
        if (!(itemStack.getItemMeta() instanceof PotionMeta)) {
            ElementOriginLib.LOGGER.warn("ItemMeta is not " + PotionMeta.class.getName());
            return this;
        }
        this.itemStack.editMeta(PotionMeta.class, (meta)-> meta.addCustomEffect(potionEffect, true));
        return this;
    }

    public ItemStackBuilder removePotionEffect(PotionEffect potionEffect) {
        if (!(itemStack.getItemMeta() instanceof PotionMeta)) {
            ElementOriginLib.LOGGER.warn("ItemMeta is not " + PotionMeta.class.getName());
            return this;
        }
        this.itemStack.editMeta(PotionMeta.class, (meta)-> meta.removeCustomEffect(potionEffect.getType()));
        return this;
    }

    public ItemStackBuilder clearPotionEffects() {
        if (!(itemStack.getItemMeta() instanceof PotionMeta)) {
            ElementOriginLib.LOGGER.warn("ItemMeta is not " + PotionMeta.class.getName());
            return this;
        }
        this.itemStack.editMeta(PotionMeta.class, (meta)-> {
            meta.clearCustomEffects();
        });
        return this;
    }

    public ItemStackBuilder addPattern(Pattern pattern) {
        if (!(itemStack.getItemMeta() instanceof BannerMeta)) {
            ElementOriginLib.LOGGER.warn("ItemMeta is not " + BannerMeta.class.getName());
            return this;
        }
        this.itemStack.editMeta(BannerMeta.class, (meta)-> meta.addPattern(pattern));
        return this;
    }

    public ItemStackBuilder removePattern(int index) {
        if (!(itemStack.getItemMeta() instanceof BannerMeta)) {
            ElementOriginLib.LOGGER.warn("ItemMeta is not " + BannerMeta.class.getName());
            return this;
        }
        this.itemStack.editMeta(BannerMeta.class, (meta)-> meta.removePattern(index));
        return this;
    }

    public ItemStackBuilder setPatterns(List<Pattern> patterns) {
        if (!(itemStack.getItemMeta() instanceof BannerMeta)) {
            ElementOriginLib.LOGGER.warn("ItemMeta is not " + BannerMeta.class.getName());
            return this;
        }
        this.itemStack.editMeta(BannerMeta.class, (meta)-> meta.setPatterns(patterns));
        return this;
    }

    public ItemStackBuilder setFireworkEffect(FireworkEffect fireworkEffect) {
        if (!(itemStack.getItemMeta() instanceof FireworkMeta)) {
            ElementOriginLib.LOGGER.warn("ItemMeta is not " + FireworkMeta.class.getName());
            return this;
        }
        this.itemStack.editMeta(FireworkMeta.class, (meta)-> meta.addEffect(fireworkEffect));
        return this;
    }

    public ItemStackBuilder clearFireworkEffects() {
        if (!(itemStack.getItemMeta() instanceof FireworkMeta)) {
            ElementOriginLib.LOGGER.warn("ItemMeta is not " + FireworkMeta.class.getName());
            return this;
        }
        this.itemStack.editMeta(FireworkMeta.class, (meta)-> meta.clearEffects());
        return this;
    }

    public ItemStackBuilder setSkullTextures(String skullTextures) {
        if (!(itemStack.getItemMeta() instanceof SkullMeta)) {
            ElementOriginLib.LOGGER.warn("ItemMeta is not " + SkullMeta.class.getName());
            return this;
        }
        this.itemStack.editMeta(SkullMeta.class, (meta)-> {
            String uuid = UUID.randomUUID().toString();

            GameProfile profile = new GameProfile(UUID.fromString(uuid), "Skull");
            profile.getProperties().put("textures", new Property("textures", skullTextures));
            Field profileField;
            try {
                profileField = meta.getClass().getDeclaredField("profile");
                profileField.setAccessible(true);
                profileField.set(meta, profile);
            } catch (NullPointerException | NoSuchFieldException | IllegalArgumentException | IllegalAccessException e1) {
            }
        });
        return this;
    }

    public ItemStackBuilder setSkullOwner(String skullOwner) {
        if (!(itemStack.getItemMeta() instanceof SkullMeta)) {
            ElementOriginLib.LOGGER.warn("ItemMeta is not " + SkullMeta.class.getName());
            return this;
        }
        this.itemStack.editMeta(SkullMeta.class, (meta)-> meta.setOwningPlayer(Bukkit.getOfflinePlayer(skullOwner)));
        return this;
    }

    public ItemStackBuilder setColor(Color color) {
        if (!(itemStack.getItemMeta() instanceof LeatherArmorMeta)) {
            ElementOriginLib.LOGGER.warn("ItemMeta is not " + LeatherArmorMeta.class.getName());
            return this;
        }
        this.itemStack.editMeta(LeatherArmorMeta.class, (meta)-> meta.setColor(color));
        return this;
    }

    public ItemStackBuilder setTrim(ArmorTrim trim) {
        if (!(itemStack.getItemMeta() instanceof ArmorMeta)) {
            ElementOriginLib.LOGGER.warn("ItemMeta is not " + ArmorMeta.class.getName());
            return this;
        }
        this.itemStack.editMeta(ArmorMeta.class, (meta)-> meta.setTrim(trim));
        return this;
    }

    private void setField(Object instance, String name, Object value) throws ReflectiveOperationException {
        Field field = instance.getClass().getDeclaredField(name);
        field.setAccessible(true);
        field.set(instance, value);
    }

}
