package com.io.yutian.elementoriginlib.util;

import com.io.yutian.elementoriginlib.exception.ItemStatDataLoadException;
import com.io.yutian.elementoriginlib.item.stat.data.*;
import org.jetbrains.annotations.NotNull;

import java.util.*;

public class StatDataUtil {

    public static BooleanData loadBooleanDataAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof Boolean bool) {
                return new BooleanData(bool);
            } else if (value instanceof String arg) {
                boolean parsed = Boolean.parseBoolean(arg);
                if (!arg.equalsIgnoreCase("true") && !arg.equalsIgnoreCase("false")) {
                    throw new ItemStatDataLoadException("Invalid boolean string: " + arg);
                }
                return new BooleanData(parsed);
            } else if (value instanceof Byte b) {
                return new BooleanData(b == 1);
            } else if (value instanceof Integer i) {
                return new BooleanData(i == 1);
            } else {
                throw new ItemStatDataLoadException("Invalid boolean value: " + value);
            }
        } catch (ClassCastException e) {
            throw new ItemStatDataLoadException("Invalid type: " + value.getClass(), e);
        }
    }

    public static DoubleListData loadDoubleListDataAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof List<?> list) {
                List<Double> doubleList = new ArrayList<>();
                for (Object item : list) {
                    if (item instanceof Double d) {
                        doubleList.add(d);
                    } else if (item instanceof String str) {
                        try {
                            doubleList.add(Double.parseDouble(str));
                        } catch (NumberFormatException e) {
                            throw new ItemStatDataLoadException("Invalid string value for Double: " + str, e);
                        }
                    } else {
                        throw new ItemStatDataLoadException("List contains non-double element: " + item);
                    }
                }
                return new DoubleListData(doubleList);
            } else if (value instanceof double[] array) {
                List<Double> doubleList = new ArrayList<>();
                for (double d : array) {
                    doubleList.add(d);
                }
                return new DoubleListData(doubleList);
            } else if (value instanceof Double[] array) {
                List<Double> doubleList = new ArrayList<>(Arrays.asList(array));
                return new DoubleListData(doubleList);
            } else if (value instanceof String str) {
                List<Double> doubleList = new ArrayList<>();
                String[] items = str.split(",");
                for (String item : items) {
                    try {
                        doubleList.add(Double.parseDouble(item.trim()));
                    } catch (NumberFormatException e) {
                        throw new ItemStatDataLoadException("Invalid number format in string: " + item, e);
                    }
                }
                return new DoubleListData(doubleList);
            } else {
                throw new ItemStatDataLoadException("Invalid value type for DoubleListData: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    public static DoubleData loadDoubleDataAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof Double d) {
                return new DoubleData(d);
            } else if (value instanceof String arg) {
                try {
                    return new DoubleData(Double.parseDouble(arg));
                } catch (NumberFormatException e) {
                    throw new ItemStatDataLoadException("Invalid double string: " + arg, e);
                }
            } else if (value instanceof Byte b) {
                return new DoubleData((double) b);
            } else if (value instanceof Integer i) {
                return new DoubleData((double) i);
            } else if (value instanceof Long l) {
                return new DoubleData((double) l);
            } else {
                throw new ItemStatDataLoadException("Invalid double value: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    public static <E extends Enum> EnumListData<E> loadEnumListDataAsObject(Class<E> clazz, @NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof List<?> list) {
                List<E> enumList = new ArrayList<>();
                for (Object item : list) {
                    if (item instanceof String str) {
                        try {
                            E enumConstant = (E) Enum.valueOf(clazz, str);
                            enumList.add(enumConstant);
                        } catch (IllegalArgumentException e) {
                            throw new ItemStatDataLoadException("Invalid enum string: " + str, e);
                        }
                    } else if (item instanceof Integer i) {
                        E[] enumConstants = clazz.getEnumConstants();
                        if (i < 0 || i >= enumConstants.length) {
                            throw new ItemStatDataLoadException("Invalid enum ordinal: " + i);
                        }
                        enumList.add(enumConstants[i]);
                    } else if (item instanceof Enum<?> enumItem) {
                        enumList.add((E) enumItem);
                    } else {
                        throw new ItemStatDataLoadException("Invalid list element type: " + item);
                    }
                }
                return new EnumListData<>(enumList);
            } else if (value instanceof String str) {
                List<E> enumList = new ArrayList<>();
                String[] enumNames = str.split(",");
                for (String enumName : enumNames) {
                    try {
                        E enumConstant = (E) Enum.valueOf(clazz, enumName.trim());
                        enumList.add(enumConstant);
                    } catch (IllegalArgumentException e) {
                        throw new ItemStatDataLoadException("Invalid enum string: " + enumName, e);
                    }
                }
                return new EnumListData<>(enumList);
            } else {
                throw new ItemStatDataLoadException("Invalid value type for EnumListData: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    public static <E extends Enum> EnumData<E> loadEnumDataAsObject(Class<E> clazz, @NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof Enum<?> enumValue) {
                return new EnumData<>((E) enumValue);
            } else if (value instanceof String str) {
                try {
                    E enumConstant = (E) Enum.valueOf(clazz, str);
                    return new EnumData<>(enumConstant);
                } catch (IllegalArgumentException e) {
                    throw new ItemStatDataLoadException("Invalid enum string: " + str, e);
                }
            } else if (value instanceof Integer i) {
                E[] enumConstants = clazz.getEnumConstants();
                if (i < 0 || i >= enumConstants.length) {
                    throw new ItemStatDataLoadException("Invalid enum ordinal: " + i);
                }
                return new EnumData<>(enumConstants[i]);
            } else {
                throw new ItemStatDataLoadException("Invalid value type for EnumData: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    public static IntListData loadIntListDataAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof List<?> list) {
                List<Integer> intList = new ArrayList<>();
                for (Object item : list) {
                    if (item instanceof Integer i) {
                        intList.add(i);
                    } else {
                        throw new ItemStatDataLoadException("List contains non-integer element: " + item);
                    }
                }
                return new IntListData(intList);
            } else if (value instanceof int[] array) {
                List<Integer> intList = new ArrayList<>();
                for (int i : array) {
                    intList.add(i);
                }
                return new IntListData(intList);
            } else if (value instanceof String str) {
                try {
                    List<Integer> intList = new ArrayList<>();
                    String[] items = str.split(",");
                    for (String item : items) {
                        intList.add(Integer.parseInt(item.trim()));
                    }
                    return new IntListData(intList);
                } catch (NumberFormatException e) {
                    throw new ItemStatDataLoadException("Invalid integer string: " + str, e);
                }
            } else {
                throw new ItemStatDataLoadException("Invalid value type for IntListData: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    public static IntData loadIntDataAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof Integer i) {
                return new IntData(i);
            } else if (value instanceof String arg) {
                try {
                    return new IntData(Integer.parseInt(arg));
                } catch (NumberFormatException e) {
                    throw new ItemStatDataLoadException("Invalid integer string: " + arg, e);
                }
            } else if (value instanceof Byte b) {
                return new IntData((int) b);
            } else if (value instanceof Long l) {
                if (l < Integer.MIN_VALUE || l > Integer.MAX_VALUE) {
                    throw new ItemStatDataLoadException("Long value out of range for integer: " + l);
                }
                return new IntData(l.intValue());
            } else if (value instanceof Double d) {
                if (d < Integer.MIN_VALUE || d > Integer.MAX_VALUE) {
                    throw new ItemStatDataLoadException("Double value out of range for integer: " + d);
                }
                return new IntData((int) d.doubleValue());
            } else if (value instanceof Float f) {
                if (f < Integer.MIN_VALUE || f > Integer.MAX_VALUE) {
                    throw new ItemStatDataLoadException("Float value out of range for integer: " + f);
                }
                return new IntData((int) f.floatValue());
            } else {
                throw new ItemStatDataLoadException("Invalid integer value: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    public static MapData loadMapDataAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof Map<?, ?> map) {
                Map<String, Object> resultMap = new HashMap<>();
                for (Map.Entry<?, ?> entry : map.entrySet()) {
                    if (entry.getKey() instanceof String key) {
                        resultMap.put(key, entry.getValue());
                    } else {
                        throw new ItemStatDataLoadException("Map key is not a String: " + entry.getKey());
                    }
                }
                return new MapData(resultMap);
            } else {
                throw new ItemStatDataLoadException("Invalid value type for MapData: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    public static StringListData loadStringListDataAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof List<?> list) {
                List<String> stringList = new ArrayList<>();
                for (Object item : list) {
                    if (item instanceof String str) {
                        stringList.add(str);
                    } else {
                        throw new ItemStatDataLoadException("List contains non-string element: " + item);
                    }
                }
                return new StringListData(stringList);
            } else if (value instanceof String[] array) {
                // 处理 String[] 类型
                return new StringListData(Arrays.asList(array));
            } else if (value instanceof String str) {
                List<String> stringList = new ArrayList<>();
                stringList.add(str);
                return new StringListData(stringList);
            } else {
                throw new ItemStatDataLoadException("Invalid value type for StringListData: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    public static StringData loadStringDataAsObject(@NotNull Object value) {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        return new StringData(value.toString());
    }

    public static UUIDData loadUUIDDataAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof UUID uuid) {
                return new UUIDData(uuid);
            } else if (value instanceof String str) {
                try {
                    return new UUIDData(UUID.fromString(str));
                } catch (IllegalArgumentException e) {
                    throw new ItemStatDataLoadException("Invalid UUID string: " + str, e);
                }
            } else {
                throw new ItemStatDataLoadException("Invalid UUID value: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

}
